'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var makeCodeAt = exports.makeCodeAt = function makeCodeAt(array) {
  return array.map(function (p) {
    return p.charCodeAt();
  });
};
var makeCodeAtRecursion = exports.makeCodeAtRecursion = function makeCodeAtRecursion(array) {
  return array.map(function (p) {
    return p instanceof Array ? makeCodeAtRecursion(p) : p.charCodeAt();
  });
};

var multiCompare = exports.multiCompare = function multiCompare(array) {
  for (var _len = arguments.length, data = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    data[_key - 1] = arguments[_key];
  }

  return array.some(function (key) {
    return data.every(function (d) {
      return d === key;
    });
  });
};
var checkInArray = exports.checkInArray = function checkInArray(data) {
  for (var _len2 = arguments.length, array = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    array[_key2 - 1] = arguments[_key2];
  }

  return array.some(function (code) {
    return code.some(function (d) {
      return d === data;
    });
  });
};

var dots = exports.dots = makeCodeAt(['。', '．']);
var lts = exports.lts = makeCodeAt(['《', '＜']);
var gts = exports.gts = makeCodeAt(['》', '＞']);
var slashs = exports.slashs = makeCodeAt(['、', '／']);
var mults = exports.mults = makeCodeAt(['×', '＊']);
var modulos = exports.modulos = makeCodeAt(['％']);
var carets = exports.carets = makeCodeAt(['…', '＾']);
var plus = exports.plus = makeCodeAt(['＋']);
var minus = exports.minus = makeCodeAt(['－', '—']);
var eqs = exports.eqs = makeCodeAt(['＝']);
var excls = exports.excls = makeCodeAt(['！']);
var amps = exports.amps = makeCodeAt(['＆']);
var pipes = exports.pipes = makeCodeAt(['｜']);
var numbers = exports.numbers = makeCodeAtRecursion([['0', '０', '零', '〇'], ['1', '１', '壹'], ['2', '２', '贰'], ['3', '３', '叁'], ['4', '４', '肆'], ['5', '５', '伍'], ['6', '６', '陆'], ['7', '７', '柒'], ['8', '８', '捌'], ['9', '９', '玖']]);

var mergedNumbers = exports.mergedNumbers = []; // eslint-disable-line import/no-mutable-exports
numbers.forEach(function (n) {
  exports.mergedNumbers = mergedNumbers = mergedNumbers.concat(n);
});
var mapping = function () {
  var table = {
    '.': dots,
    '<': lts,
    '>': gts,
    '/': slashs,
    '*': mults,
    '%': modulos,
    '^': carets,
    '+': plus,
    '-': minus,
    '=': eqs,
    '!': excls,
    '&': amps,
    '|': pipes,
    /* eslint quote-props: 0 */
    '0': numbers[0],
    '1': numbers[1],
    '2': numbers[2],
    '3': numbers[3],
    '4': numbers[4],
    '5': numbers[5],
    '6': numbers[6],
    '7': numbers[7],
    '8': numbers[8],
    '9': numbers[9]
  };
  var w = {};
  Object.keys(table).forEach(function (key) {
    table[key].forEach(function (d) {
      w[d] = key;
    });
  });
  return w;
}();
exports.default = mapping;